% This simulator runs the main control loop
% Not only does it update the simulated robot's position and sensor
% measurements, but it conducts localization, path planning, and path
% tracking.
function [X] = arw_simulator(X_0, X_des)
    
    % Initialize variables
    [ X_real, X_est, U, pathTracked, timeStep, M, P, T, e, nodes, deltaT ] = initializeVariables( X_0 );
    
    % Loop over time, until pathtracking is complete
    maxTimeStep = 50;
    while pathTracked == false && timeStep <= maxTimeStep
        
        % Simulate the actual robot to get the real state, measured
        % odometry O, and range measurements Z
        [ X_real, O, Z ] = simulateMotionAndPerception( X_real, U, M, deltaT );
               
        % Localize the robot, i.e. estimate the current state X
        [X_est, P] = localizeRobot(X_est, O, Z, M, P);
        
        % Construct a new path if it is the first iteration
        [T, nodes] = constructPath(X_0, X_des, M, T, nodes);
        
        % Determine the control inputs U to track the path T
        [U, T, pathTracked] = pathTrackingControl(X_est, T);
        
        % Save states and calculate the error e
        X(timeStep,1:3) = X_real;
        e(timeStep) = sqrt((X_real(1)-X_est(1))^2 + (X_real(2)-X_est(2))^2);
        
        % Plot the output
        plotState(X_real, e, X, Z, M, P, nodes, T);
        timeStep = timeStep + 1; 
    end
end























%%%%%%%%%%%%%%%%%%%%%%%%%%%% MAP FUNCTIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%








