% This function is called by the perception simulator, to calculate range
% sensor measurementslocalization algorithm, as well as by the localization
% algorithm, to calculate expected range measurements.

% The function loops through all walls and finds the closest one by calling
% getRangeToWall
function [minRange] = getClosestRangeToWalls(x, y, theta, M)
    minRange = 999999;
    for i=1:size(M,1)

        range = getRangeToWall(x, y, theta, M(i,1:size(M,2)));
        if range < minRange
            minRange = range;
        end
    end
end

% This function finds the distance to a wall if the range measurement 
% actually reflects off the wall. The math finds the intersection of two
% lines, one that passes along the wall segment, and one that is in the
% diretion that the robot sensor is pointing.
function [range] = getRangeToWall(x, y, theta, wall)
    range = 10.0;

    % Set wall vars
	X1 = wall(1);
	Y1 = wall(2);
	X2 = wall(3);
	Y2 = wall(4);
    L2_w = wall(5)^2;%sqrt((X1-X2)^2+(Y1-Y2)^2);
    m_w = wall(6);
    b_w = wall(7);
    
	%Range theta
    theta = angleDiff(theta);
	
	% Calculate slope and intercept
	m = tan(theta);
	b = y - m*x;

	% Get line intersection
	x_intersect = (b-b_w)/(m_w-m);
	y_intersect = m*x_intersect + b;
	angle_intersect = atan2(y_intersect-y, x_intersect-x);

	% Check to see if intersection LIES within segment
	dist_intersect_corner1_2 = ((x_intersect-X1)^2 + (y_intersect-Y1)^2);
	dist_intersect_corner2_2 = ((x_intersect-X2)^2 + (y_intersect-Y2)^2);
	if (dist_intersect_corner1_2 <= L2_w) && (dist_intersect_corner2_2 <= L2_w) && (abs(theta-angle_intersect)<0.01)
		range = sqrt((x-x_intersect)^2 + (y-y_intersect)^2);
    end
end
