% This function is called by the main arw_simulator to initialize all
% variables.
% deltaT is the amount of time (s) taken between each control loop update
% unknownStart is used to set whether the robot begins by knowing where it
% is or not. It is used to determine if particles are initially spread 
% everywhere, or at the known start location.
function [ X_real, X_est, U, pathTracked, timeStep, M, P, T, e, nodes, deltaT ] = initializeVariables( X_0 )
    X_real = X_0;
    X_est = X_0;
    U = [0 0];
    pathTracked = false;
    timeStep=1;
    M = setMap();
    deltaT = 0.1;
    numParticles = 50;
    unknownStart = false;
    [P] = initializeParticles(X_0, M, numParticles, unknownStart);
    T=[];
    e=[];
    nodes=[];
end

% Spread the particles randomly but within bounds, OR assume the particels 
% are at a known position.
function [P] = initializeParticles(X, M, numParticles, unknownStart)
    P = zeros(numParticles,3);
    if unknownStart == true
        mapRanges = getMapRanges(M);
        P(:,1) = mapRanges(1)*ones(numParticles,1) + rand(numParticles,1)*(mapRanges(2)-mapRanges(1)); 
        P(:,2) = mapRanges(3)*ones(numParticles,1) + rand(numParticles,1)*(mapRanges(4)-mapRanges(3)); 
        P(:,3) = -pi*ones(numParticles,1) + rand(numParticles,1)*2*pi; 
    else
        P(:,1) = X(1); 
        P(:,2) = X(2); 
        P(:,3) = X(3); 
    end
end





function [M] = setMap()

    maxNumWalls = 8;
    M = zeros(maxNumWalls, 7);
    s=3;
    M(1,1:4) = [s/2 s s s];
    M(2,1:4) = [-s/2 s -s s];
    M(3,1:4) = [-s s -s -s];
    M(4,1:4) = [s -s -s -s];
    M(5,1:4) = [s -s s s];
    M(6,1:4) = [0 -s 0 -s/2];
    M(7,1:4) = [0 -s/2 -s/2 -s/2];
    M(8,1:4) = [s/2 s s/2 s/4];
    
    
    for i=1:maxNumWalls
        M(i,5) = sqrt((M(i,1)-M(i,3))^2+(M(i,2)-M(i,4))^2);
        if M(i,3)== M(i,1)
            M(i,6) = 999999;
        else
            M(i,6) = (M(i,4)-M(i,2))/(M(i,3)-M(i,1));
        end
        M(i,7) = M(i,2)-M(i,6)*M(i,1);
    end
    
    %plotMap(M);
end

function [mapRanges] = getMapRanges(M)
      
    minX = min(min(M(:,1)),min(M(:,3)));
    maxX = max(max(M(:,1)),min(M(:,3)));
    minY = min(min(M(:,2)),max(M(:,4)));
    maxY = max(max(M(:,2)),max(M(:,4)));
    mapRanges = [minX maxX minY maxY];
end