%%%%%%%%%%%%%%%%%%%%%%%%%%%% CONTROL FUNCTIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%


% This function will determine which point on a path T should be tracked 
% comparing the robot's current position with the next untrack point on T
% After determining the point to track, it calls the pointTrackingControl
% function to be written by ARW participants.
function [U, T, pathTracked] = pathTrackingControl(X_est, T)
    
    % Set thresholds
    posTrackingThreshold = 0.2;
    angTrackingThreshold = 0.3;
    trackingThresholds = [posTrackingThreshold angTrackingThreshold];

    % Set pathFound
    pathTracked = false;
    
    % Determine point to track    
    % Check if second last point of path has been tracked, to just track
    % the final point
    if T(size(T,1)-1,4) == 1
        X_toTrack = T(size(T,1),1:3);
        if withinThreshold(X_est, T(size(T,1),1:3), trackingThresholds) 
            pathTracked = true;
        end
         
    % Otherwise, find the current point being tracked and see if acheived    
    else
        for i=1:size(T,1)
            if T(i,4) == 0
                if withinThreshold(X_est, T(i,1:3),trackingThresholds)
                   T(i,4) = 1;
                   X_toTrack = T(i+1,1:3);
                else
                   X_toTrack = T(i,1:3);
                end
                break;
            end
        end
    end
    
    % Call point tracking controller to 
    U_inRad = pointTrackingControl(X_toTrack, X_est, trackingThresholds);
    
    % Converting from rad/s to pulses/s
    pulsesPerRevolution = 4096;
    radToPulses = pulsesPerRevolution/(2*pi);
    U = U_inRad*radToPulses;
end

% A helper function to check if two robot states are within threshold distance
% and direction of each other.
function pointTracked = withinThreshold(X1, X2, trackingThresholds)
    
    pointTracked = false;
    dist2 = ((X1(1)-X2(1))^2+(X1(2)-X2(2))^2);
    ang = abs(angleDiff(X1(3)-X2(3)));
    if dist2 < trackingThresholds(1)^2 && ang < trackingThresholds(2)
        pointTracked = true;
    end

end

% For this controller to work, ARW participants must write the point
% tracking controller presented in lecture. Inputs include the desired
% state, current estimated state, and threshold values that determines if
% the states are close enough to be considered tracked.
%The output of the function is the right and left wheel speed in pulses / second.
function U = pointTrackingControl(X_toTrack, X_est, trackingThresholds)

    % Some useful variables
    K_rho = 1.0;
    K_alpha = 2.0;
    K_beta = -0.99;
    robotRadius = 0.2;
    robotWidth = robotRadius*2;
    wheelRadius = 0.1;

    % Some helpful calculations
    deltaX = X_toTrack(1) - X_est(1);
	deltaY = X_toTrack(2) - X_est(2);
	ang = atan2(deltaY,deltaX);
    
    % Calucate rho, the distance to goal
	% rho = ...
			
	% Check to see if simply rotation on the spot, if rho is small
    % if ...
        % rho = ...
        % alpha = ...
        % beta = ...
    
    
    % if rho is still too big, invoke the point tracking theory
    % else ...

        % Calculate alpha and beta
        % alpha = ...
        % beta = ...
    
    
    
    % Calculate v, w
    % v = ...
    % w = ....
    
    
    
    % Calculate desired wheelSpeed in pulses / second
    %desiredWheelSpeedR = ...
    %desiredWheelSpeedL = ...
    
    

    
    
    
    
	% Check that max velocity in rad/s isn't reached
    currentMaxVel = max(abs(desiredWheelSpeedL), abs(desiredWheelSpeedR));
    maxWheelSpeed = 2*pi;
    if  currentMaxVel > maxWheelSpeed
        desiredWheelSpeedR = desiredWheelSpeedR * maxWheelSpeed / currentMaxVel;
        desiredWheelSpeedL = desiredWheelSpeedL * maxWheelSpeed / currentMaxVel;
    end
    
    % Set the control vector U
    U = [desiredWheelSpeedR desiredWheelSpeedL];

end


