% This function calculates both the odometry (wheel encoder pulses)
% and the range measurements. The function is called directly by the
% arw_simulator to update the robot's simulated measurements at every
% time step
function [O, Z] = perception(X, U, M, deltaT)

    % Get the odometry, a 1x2 vector of encoder pulse changes
    % U is rotational velocity of each wheel in pulses / sec
    deltaEncoderR = U(1)*deltaT;
    deltaEncoderL = U(2)*deltaT;
    O=[deltaEncoderR deltaEncoderL];
    
    % Get the range measurements
    minSensorAngle = -pi*120/180;
    sensorAngleResolution = 20*pi/180;
    sensorFieldOfView = 240*pi/180;
    maxNumSensorAngles = ceil(sensorFieldOfView/sensorAngleResolution);
    
    % First entry of each row of Z is the range bearing, second is the range.
    Z = zeros(maxNumSensorAngles,2);
    for i=1:maxNumSensorAngles
       Z(i,1) = minSensorAngle + i*sensorAngleResolution;
       Z(i,2) = getClosestRangeToWalls(X(1),X(2),angleDiff(X(3) + Z(i,1)), M);
    end

end
