%%%%%%%%%%%%%%%%%%%%%%%%%%%% PLOTTING FUNCTIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%
function [] = plotState(X_real, e, X, Z, M, P, nodes, T)
    R = 0.2;
    figure(1);
       
    set( gcf, 'DoubleBuffer', 'on' );
    low = min(min(M(:,1:4)))-2;
    high = max(max(M(:,1:4)))+2;
    axisVector = [low high low high];
        
    % Plot the robot and workspace in top 90% of Figure
    g=subplot(10,1,[1:9],'Color','Black');
    cla(g);
    hold on;

    % plot the RRT nodes if you want to double check the RRT works
    %plotNodes(nodes,axisVector);
    plotPath(T, axisVector);

    % Plot the robot as a circle with direction line
    plot([X_real(1) X_real(1)+R*cos(X_real(3))],[X_real(2) X_real(2)+R*sin(X_real(3))],'w');
    axis(axisVector);
    circle(X_real(1),X_real(2),R, axisVector); 
    
    % Plot the map, sensor data, and particles used for localization
    plotMap(M, axisVector);
    plotRangeData(X_real, Z, R, axisVector);
    plotParticles(P, axisVector);
    drawnow;

    % In the bottome 10% of the figure, plot the localization eeror.
    subplot(10,1,10);
    %set( gcf, 'DoubleBuffer', 'on' );
    plot(e);
    xlabel('Time Step');
    ylabel('Error (m)');
    drawnow;
    
    
end

function [] = circle(x,y,r, axisVector)
    ang=0:0.1:2*pi; 
    xp=r*cos(ang);
    yp=r*sin(ang);
    plot(x+xp,y+yp,'w');
    axis(axisVector);
end

function [] = plotMap(M, axisVector)
    figure(1);
    hold on;
    for i=1:size(M,1)
       plot([M(i,1) M(i,3)], [M(i,2) M(i,4)], 'w');
    end
    axis(axisVector);
end

function []= plotRangeData(X, Z, R, axisVector)
    figure(1);
    hold on;
    
    for i=1:size(Z,1)
        X1 = X(1) +R*cos(X(3)+Z(i,1));
        Y1 = X(2) +R*sin(X(3)+Z(i,1));
        X2 = X(1) +Z(i,2)*cos(X(3)+Z(i,1));
        Y2 = X(2) +Z(i,2)*sin(X(3)+Z(i,1));
        
       plot([X1 X2],[Y1 Y2], 'y');
       axis(axisVector);
       
    end
    drawnow;
end

function [] = plotParticles(P, axisVector)

    for i=1:size(P,1)
       plot(P(i,1),P(i,2),'r.'); 
       axis(axisVector);
       
    end
drawnow;
end



function [] = plotPath(T, axisVector)
    figure(1);
    plot(T(:,1),T(:,2),'r');
    axis(axisVector);
end

function [] = plotNodes(nodes, axisVector)
    figure(1);
    for i=2:size(nodes,1)
        plot([nodes(i,1) nodes(nodes(i,3),1)],[nodes(i,2) nodes(nodes(i,3),2)],'c');
        axis(axisVector);
    end
end
