% This function is called at every iteration of the main control loop in 
% the arw_simulator  file. It 1) determines the change in wheel encoder
% values, 2) updates the robot state using kinematic equations, and 3)
% calls perception to update the robot's sensor measurements.
function [ X_real, O, Z ] = simulateMotionAndPerception( X_real, U, M, deltaT )
        
    % Simulate the number of encoder pulse changes for the rotational
    % velocity of each wheel in units of pulses/sec.
    deltaEncoderR = U(1)*deltaT;
    deltaEncoderL = U(2)*deltaT;

    % Set State Vector
    X_real = kinematicModel(X_real, deltaEncoderR, deltaEncoderL);

    % Simulate the laser range measurements Z and the odometry O
    % given the new state of the robot and the control inputs U
    [O, Z] = perception(X_real, U, M, deltaT);

end



